import { buffer_tx } from '@scrow/tapscript/tx';
import { CoreWallet } from './wallet.js';
import { parse_args, run_cmd } from './cmd.js';
import { cmd_config, core_config } from '../config.js';
export class CoreClient {
    constructor(core, config) {
        const opt = core_config(config);
        const { debug } = opt;
        this._cache = ['null', null];
        this.params = [
            `-chain=${opt.network}`,
            `-rpcport=${opt.rpc_port}`,
            ...opt.params,
            ...opt.cli_params
        ];
        if (opt.confpath !== undefined) {
            this.params.push(`-conf=${opt.confpath}`);
        }
        if (opt.datapath !== undefined) {
            this.params.push(`-datadir=${opt.datapath}`);
        }
        if (opt.cookiepath !== undefined) {
            this.params.push(`-rpccookiefile=${opt.cookiepath}`);
        }
        this._opt = opt;
        this._core = core;
        this._faucet = null;
        if (debug)
            console.log('[debug] Initializing CLI with params:', this.params.join(' '));
    }
    get opt() {
        return this._opt;
    }
    get chain_info() {
        return this.cmd('getblockchaininfo');
    }
    get core() {
        return this._core;
    }
    get wallets_loaded() {
        return this.cmd('listwallets', null, { cache: true });
    }
    get wallets_created() {
        return this.cmd('listwalletdir', null, { cache: true })
            .then(e => e.wallets.map(x => x.name));
    }
    async cmd(method, args, config) {
        const { clipath = 'bitcoin-cli', debug } = this.opt;
        const { cache, params } = cmd_config(config);
        const parsed = parse_args(method, args);
        const witness = [...this.params, ...params, ...parsed];
        const label = witness.join('');
        if (debug) {
            const offset = this.params.length;
            console.log('[client] cmd:', witness.slice(offset).join(' '));
        }
        if (cache && this._cache[0] === label) {
            if (debug)
                console.log('[client] using cache for method:', method);
            return this._cache[1];
        }
        const data = await run_cmd(clipath, witness);
        this._cache = [label, data];
        return data;
    }
    async _get_block_data(query, txdata = false) {
        let { height, hash } = query;
        if (height === undefined && hash === undefined) {
            height = await this.cmd('getblockcount');
        }
        if (typeof height === 'number') {
            hash = await this.cmd('getblockhash', height);
        }
        if (typeof hash !== 'string') {
            throw new Error('Unable to fetch any blocks!');
        }
        return (txdata === true)
            ? this.cmd('getblock', hash)
            : this.cmd('getblockheader', hash);
    }
    async get_block(query) {
        return this._get_block_data(query, true);
    }
    async get_header(query) {
        return this._get_block_data(query, false);
    }
    async scan_txout(action, ...desc) {
        return this.cmd('scantxoutset', [action, JSON.stringify(desc)]);
    }
    async scan_addr(addr) {
        return this.scan_txout('start', `addr(${addr})`).then(res => {
            const { success, unspents } = res;
            return (success) ? unspents : [];
        });
    }
    async mine_blocks(count = 1, addr) {
        if (this.opt.network !== 'regtest') {
            throw new Error('You can only generate funds on regtest network!');
        }
        if (addr === undefined) {
            addr = await this.core.faucet.get_address('faucet');
        }
        return this.cmd('generatetoaddress', [count, addr]);
    }
    async get_tx(txid) {
        let txdata, status = { confirmed: false };
        try {
            txdata = await this.cmd('getrawtransaction', [txid, true], { cache: true });
        }
        catch (err) {
            return null;
        }
        const { hex, ...data } = txdata;
        const block_hash = txdata.blockhash;
        const block_time = txdata.blocktime;
        if (block_hash !== undefined && block_time !== undefined) {
            const { height } = await this.get_header({ hash: block_hash });
            status = { confirmed: true, block_height: height, block_hash, block_time };
        }
        return { data, hex, status };
    }
    async load_wallet(name) {
        const wallet = new CoreWallet(this, name);
        await wallet.load();
        await wallet.init();
        return wallet;
    }
    async load_wallets(...labels) {
        const wallets = {};
        const files = await this.wallets_created;
        const names = await this.wallets_loaded;
        for (const label of labels) {
            const wallet = new CoreWallet(this, label);
            if (!names.includes(label)) {
                if (!files.includes(label)) {
                    await wallet._create();
                }
                else {
                    await wallet._load();
                }
            }
            wallets[label] = wallet;
        }
        return wallets;
    }
    async publish_tx(txdata, confirm = false) {
        const txhex = buffer_tx(txdata).hex;
        const txid = await this.cmd('sendrawtransaction', [txhex]);
        if (confirm)
            await this.mine_blocks(1);
        return txid;
    }
}
//# sourceMappingURL=client.js.map